#include "SWM221.h"
#include "CircleBuffer.h"
#include "jw_can.h"
#include "jw_bsp_init.h"
#include "jw_sys_timer.h"

/* 测试条件：SWM221 <==CAN_RX/TX==> CAN 收发器（如 TJA1050）<==CAN_H/L==> CAN 分析仪
 * 错误处理演示：
 *	1、将 CAN 收发器与 CAN 分析仪断开，触发 ACK 错误，CAN->TXERR 从 0 递增到 128，然后保持不变
 *	2、将 SWM221 与 CAN 收发器断开，触发位错误，CAN->TXERR 继续递增到 255，触发 Bus Off，CAN->CR.RST 自动置 1，CAN 模块处于复位状态
 *	3、ISR 中在检测到 Bus Off 后执行 CAN_Open(CAN0) 使 CAN 模块退出复位状态，此时将上面断开的连接恢复，CAN->TXERR 将逐渐递减到0，CAN 模块恢复正常工作
*/
static can_data_t can_buff[CAN_SIZE_MAX] = {	
	/* MCU ID值, cnt-->11 */
	{.can_id = 0x101,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//参数设置
	{.can_id = 0x102,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//功能开关设置
	{.can_id = 0x103,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//功能控制
	{.can_id = 0x104,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//信息获取
	{.can_id = 0x500,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//版本信息
	{.can_id = 0x501,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//序列号
	{.can_id = 0x502,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//系统信息1	/*周期：1000ms*/
	{.can_id = 0x503,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//系统信息2	/*周期：1000ms*/
	{.can_id = 0x600,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//故障状态	/*周期：500ms*/
	{.can_id = 0x601,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//运行信息1	/*周期：200ms*/
	{.can_id = 0x602,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//功率输出状态	/*周期：500ms*/
	/* BMS ID值, cnt-->21 */	
	{.can_id = 0x1000FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//版本信息					/*周期：2000ms*/
	{.can_id = 0x1001FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//厂商信息					/*周期：2000ms*/
	{.can_id = 0x1002FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包生产日期			/*周期：2000ms*/
	{.can_id = 0x1003FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包BMS序列号			/*周期：2000ms*/
	{.can_id = 0x1004FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包额定参数			/*周期：2000ms*/
	{.can_id = 0x1010FFA7,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包工作状态--汇总信息	/*周期：100ms*/
	{.can_id = 0x1011FFA7,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包电压电流--汇总信息	/*周期：100ms*/
	{.can_id = 0x1012FFA7,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包SOP--汇总信息		/*周期：100ms*/
	{.can_id = 0x1020FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包电压及MOS信息		/*周期：100ms*/
	{.can_id = 0x1021FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包瞬时充放电电流		/*周期：100ms*/
	{.can_id = 0x1022FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包绝缘检测信息		/*周期：500ms*/
	{.can_id = 0x1023FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包SOX估算信息			/*周期：500ms*/
	{.can_id = 0x1024FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包剩余容量信息		/*周期：500ms*/
	{.can_id = 0x1025FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包单体电压信息		/*周期：500ms*/
	{.can_id = 0x1026FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包持续电流信息		/*周期：500ms*/
	{.can_id = 0x1027FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包温度信息			/*周期：500ms*/
	{.can_id = 0x1028FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包故障信息			/*周期：500ms*/
	{.can_id = 0x1030FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包单体电压信息		/*周期：500ms*/
	{.can_id = 0x1031FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包温度信息			/*周期：500ms*/
	{.can_id = 0x1032FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包实时时间信息		/*周期：500ms*/
	{.can_id = 0x1033FFA0,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//电池包Pack SN码			/*周期：100ms*/
	/* VCU ID值, cnt-->4 */
	{.can_id = 0x630,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//系统状态		/*周期：200ms*/
	{.can_id = 0x631,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//辅助功能状态	/*周期：500ms*/
	{.can_id = 0x632,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//防盗			/*周期：500ms*/
	{.can_id = 0x130,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//控制指令			
	/* HIMI ID值, cnt-->2 */
	{.can_id = 0x650,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//状态信息	/*周期：500ms*/
	{.can_id = 0x150,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//参数设置
	/* TBOX ID值, cnt-->1 */
	{.can_id = 0x640,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//状态信息	/*周期：500ms*/
	/* LED ID值, cnt-->2 */
	{.can_id = 0x200,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} },	//beam_control	/*周期：100ms*/
	{.can_id = 0x201,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} }	//beam_set	/*周期：100ms*/	
};

//can_data_t radio_can_buff[CAN_SIZE_MAX]  = {
//	/*	VCU广播地址	*/
//	.can_id = 0x00282301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x00292301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x002A2301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x002B2301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x002C2301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},	
//	.can_id = 0x002D2301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x002E2301,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},	
//	.can_id = 0x00282302,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x00292302,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	.can_id = 0x002A2302,.data = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
//	

//};

volatile bool msg_rcvd = false;

CircleBuffer_t CirBuf;
CircleBuffer_t TX_CirBuf;

void SerialInit(void);
void  UART_send(void);
void  CAN_send_id(int id,uint8_t *pdat);

//#define LOG printf
static uint8_t m_buf[1024];

/**
  * @brief   在CAN数据库中查找指定ID的数据地址 
  * @param   id : 要查找的CAN消息ID 
  * @retval  成功返回对应数据的地址指针，失败返回NULL 
  */
static __inline uint8_t *find_can_db(int id)
{
    int i = 0, idx = 0;              // 循环计数器i，匹配索引idx 
    can_data_t *p_arry = can_buff; // 获取CAN数据库首地址 
    
    // 遍历整个CAN数据库 
    for (; i < CAN_SIZE_MAX; i++) 
    { 
        // 比对当前项的CAN ID 
        if (p_arry[i].can_id == id) { 
            idx = i;  // 记录匹配项的索引 
            break;    // 找到后立即退出循环 
        } 
    }
    
    // 返回结果：如果遍历完未找到返回NULL，否则返回对应数据的地址 
    return (i == CAN_SIZE_MAX) ? NULL : p_arry[idx].data;
}

int main(void)
{
	int len=0,i;
	uint8_t *pcan_dat = NULL;        // 定义指向CAN数据的指针
	can_data_t can_dat = {0};        // 定义CAN数据结构体并初始化为0

	bsp_init();
	segger_rtt_init("	[JW LOG]:---->>>> SG01 app start ! <<<----\r\n"); // 初始化SEGGER RTT调试日志
	
	while(1)  // 嵌入式系统典型的主循环（Super Loop）结构 
	{
//		//原CAN发送代码（已注释，保留供调试参考）
//		uint8_t tx_data[8] = {1, 2, 3, 4, 5, 6, 7, 8},i;
//		CAN_Transmit(CAN0, CAN_FRAME_STD, 0x104, tx_data, 8, 1);
//		while(CAN_TXComplete(CAN0) == 0) __NOP(); // 阻塞等待CAN发送完成 
//		printf("\r\nCAN->TXERR: %d\r\n", CAN0->TXERR); // 调试用：打印CAN发送错误计数 
//		for(i=0;i<9;i++)
//		{
//			UART_WriteByte(UART0, tx_data[i]); // 写入UART发送寄存器 
//			LOG(">>>>>>send  data :%d  <<<<<<\r\n",tx_data[i]);			
//		}
		
		/*	核心逻辑：UART消息处理与转发	*/
		if(msg_rcvd)  // 检测UART接收完成标志（由UART中断置位）
		{
			msg_rcvd = false;  // 立即清除标志，避免重复处理 
	 
			// 检查环形缓冲区数据有效性 
			if(!CirBuf_Empty(&CirBuf)) 
			{
				len = CirBuf_Count(&CirBuf);  // 获取缓冲区数据长度 
	 
				// 数据长度校验（12~1023字节为合法范围）
				if( (len < 12) || (len > (1024-1)) ) 
				{
					CirBuf_Clear(&CirBuf);  // 非法数据清空缓冲区 
					LOG(">>>>>>CAN data len:%d ERR! <<<<<<\r\n",len); // 记录错误日志 
				}
				else 
				{   
					/*错：此处串口发送的数据不对，需要修改*/
					// 有效数据处理流程：
					CirBuf_Read(&CirBuf, (uint8_t *)m_buf, len);  // 读取数据到m_buf 
					CAN_send_id((int)m_buf, &m_buf[4]);  // 解析并发送CAN帧（假设前4字节为ID，后续为数据）
				}
			}
	 
			// 启用UART发送中断（准备后续数据发送）
			UART_INTEn(UART0, UART_IT_TX_THR); 
		}
		
//		/*--- CAN数据处理部分 ---*/
//        for(i = 0; i < CAN_SIZE_MAX; i++) // 遍历CAN数据库
//        {
//			 /*--- 常规CAN数据处理 ---*/
//            pcan_dat = find_can_db(can_dat.can_id);  // 在CAN数据库查找对应ID
//            if( pcan_dat == NULL )
//            {
//                //LOG(">>>>>>>>>>>>> Not Find CAN id %08X <<<<<<<<<<<<<\n",can_dat.can_id);
//                break;
//            }
//            memcpy(pcan_dat,can_dat.data,sizeof(can_dat.data)); // 拷贝数据到数据库
//		}
		
		// 主动触发UART发送（非中断驱动部分）
		UART_send();
		
		/*--- 定时任务处理 ---*/
		//Timer_MainFun();
		
		// 延时控制（基于系统时钟的精确空操作延时） 
		for(int i = 0; i < SystemCoreClock / 16; i++) __NOP(); // 约62.5ms延时（SystemCoreClock=8MHz）
	}
}


void CAN0_Handler(void)
{
	uint32_t can_if = CAN_INTStat(CAN0);
	uint32_t id_p = 0; //id位置互换
	
	// 处理接收数据可用中断
	if(can_if & CAN_IF_RXDA_Msk)
	{
		CAN_RXMessage msg;
		
		CAN_Receive(CAN0, &msg);
		
		if(msg.size > 0)
		{
			LOG_ORG("	Receive %s: %08X", msg.format == CAN_FRAME_STD ? "STD" : "EXT", msg.id);			
			for(int i = 0; i < msg.size; i++) 
				LOG_ORG(" %02X", msg.data[i]);
			LOG("\n");
			if(msg.format == CAN_FRAME_STD)
			{		
				id_p = (msg.id <<24)&0xff000000;
				id_p |= (msg.id <<8)&0x00ff0000;
				CirBuf_Write(&TX_CirBuf, (uint8_t *)&id_p, 4);
				CirBuf_Write(&TX_CirBuf, (uint8_t *)msg.data, 8);
			}
			else
			{				
				id_p = (msg.id <<24)&0xff000000;
				id_p |= (msg.id <<8)&0x00ff0000;
				id_p |= (msg.id >>24)&0x000000ff;
				id_p |= (msg.id >>8)&0x0000ff00;
				CirBuf_Write(&TX_CirBuf, (uint8_t *)&id_p, 4);
				CirBuf_Write(&TX_CirBuf, (uint8_t *)msg.data, 8);
			}				
		}
		else if(msg.remote == 1)	//远程帧
		{
			LOG_ORG("	Receive %s Remote Request\r\n", msg.format == CAN_FRAME_STD ? "STD" : "EXT");
		}
	}
	
	// 处理接收溢出中断 
	if(can_if & CAN_IF_RXOV_Msk)
	{
		LOG_ORG("	CAN RX Overflow\r\n");
		
		// 关闭CAN0模块以清除溢出状态 
		CAN_Close(CAN0);
		//延时确保CAN完全关闭
		for(int i = 0; i < CyclesPerUs; i++) __NOP();
		// 重新打开CAN0模块以恢复通信 
		CAN_Open(CAN0);
	}
	
	// 处理错误警告中断
	if(can_if & CAN_IF_ERRWARN_Msk)
	{
		// 检查是否发生总线关闭错误 
		if(CAN0->SR & CAN_SR_BUSOFF_Msk)
		{
			LOG("	CAN Bus Off\r\n");
			LOG_ORG("	CAN->CR.RST = %d\r\n", CAN0->CR & CAN_CR_RST_Msk ? 1 : 0);
			
			// 尝试恢复总线连接
			CAN_Open(CAN0);			//Bus Off recovery
		}
		else if(CAN0->SR & CAN_SR_ERRWARN_Msk)
		{
			LOG("	CAN Error Warning\r\n");
		}
	}
	
	// 处理错误被动中断 
	if(can_if & CAN_IF_ERRPASS_Msk)
	{
		LOG("	CAN Error Passive\r\n");
	}
}

 void  CAN_send_id(int id,uint8_t *pdat)
 {
 	int can_id = id;
 	uint8_t data[8] = {0};
 	memcpy(data,pdat,8);

 	CAN_Transmit(CAN0, CAN_FRAME_STD, can_id, data, 8, 1); 
 	while(CAN_TXComplete(CAN0) == 0) __NOP();
 	if( CAN0->TXERR  )	
 	{
 		LOG(">>>>>>>>>>>>>>>>>>CAN Error CAN->TXERR: %d<<<<<<<<<<<<<<<<<<<<<\r\n", CAN0->TXERR);	
 	}
 }

/**
 * @brief UART0中断服务函数 
 * 处理UART0的接收和发送中断，包括数据缓冲和状态管理 
 */
void UART0_Handler(void)
{
    uint32_t chr; // 临时存储读取/发送的字符 
    
    /* 接收中断处理（RX FIFO阈值中断或接收超时中断）*/
    if(UART_INTStat(UART0, UART_IT_RX_THR | UART_IT_RX_TOUT)) 
    {
        // 循环读取RX FIFO直到为空 
        while(UART_IsRXFIFOEmpty(UART0) == 0)
        {
            // 成功读取字节时写入环形缓冲区 
            if(UART_ReadByte(UART0, &chr) == 0)
            {
                CirBuf_Write(&CirBuf, (uint8_t *)&chr, 1); // 写入接收缓冲区 
				LOG("	uart RX... \r");
            }
        }
        
        /* 特殊处理接收超时中断（数据流间隔触发）*/
        if(UART_INTStat(UART0, UART_IT_RX_TOUT))
        {
            UART_INTClr(UART0, UART_IT_RX_TOUT); // 清除中断标志 
            msg_rcvd = true; // 设置消息接收完成标志（可能用于主程序轮询）
			LOG("	uart RX TOUT... \r");
        }
    }
    
    /* 发送中断处理（TX FIFO阈值中断）*/
    if(UART_INTStat(UART0, UART_IT_TX_THR))
    {
        // 当TX FIFO未满时持续发送数据 
        while(!UART_IsTXFIFOFull(UART0))
        {
            // 如果发送缓冲区有数据则取出并发送 
            if(!CirBuf_Empty(&TX_CirBuf))
            {
                CirBuf_Read(&TX_CirBuf, (uint8_t *)&chr, 1); // 从发送缓冲区读取 
                UART_WriteByte(UART0, chr); // 写入UART发送寄存器 
				LOG("	uart TX...	 \r");
            }
            else 
            {
                // 发送缓冲区为空时禁用TX中断（避免空转）
                UART_INTDis(UART0, UART_IT_TX_THR);
				LOG("	uart ERR...	 \r");
                break; // 退出发送循环 
            }
        }
    }
} 	

void UART_send(void)
{
	uint32_t chr = 0;
	// if(UART_INTStat(UART1, UART_IT_TX_THR))//not chk TX_THR
	{
		while(!UART_IsTXFIFOFull(UART0))
		{
			if(!CirBuf_Empty(&TX_CirBuf))
			{
				CirBuf_Read(&TX_CirBuf, (uint8_t *)&chr, 1);				
				UART_WriteByte(UART0, chr);
				while(UART_IsTXBusy(UART0));	//发送空闲跳出
				// while(UART_IsTXBusy(UART1))__NOP();
				// LOG("[ UART_send ] TX ->: %02X\n", chr&0xFF); 
				//LOG("handle TX msg_rcvd->: %d   cnt:%d \r\n", msg_rcvd,CirBuf_Count(&CirBuf)); 
			}
			else
			{
				UART_INTDis(UART0, UART_IT_TX_THR);
				//LOG("~~~~~~~~Cycle TX buffer Empty~~~~~~~\n");
				break;
			}
		} 
	}
}


/*********************************************************************************************
* 函数名称: fputc()
* 功能说明: printf()使用此函数完成实际的串口打印动作
* 输    入: int ch		要打印的字符
*			FILE *f		文件句柄
* 输    出: 无
* 注意事项: 无
**********************************************************************************************/
int fputc(int ch, FILE *f)
{
#if 0
	UART_WriteByte(UART0, ch);
	
	while(UART_IsTXBusy(UART0));
#endif 	


	return ch;
}
